#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from collections import defaultdict

class NotificationCenter(object):
    _listeners = None

    def __new__(type):
        if not '_singleton' in type.__dict__:
            type._singleton = object.__new__(type)
            type._singleton._listeners = defaultdict(list)

        return type._singleton

    def on(self, etype, listener):
        self._listeners[etype].append(listener)

    def off(self, etype, listener=None):
        # self._listeners.remove(etype)
        pass

    def fire(self, notification):
        # 前置
        preEvent = notification.pre()
        if preEvent:
            for l in self._listeners[preEvent.type()]:
                if l(notification): # 如果返回 True ，那么消息不再传递下去
                    break;

        for l in self._listeners[notification.type()]:
            if l(notification): # 如果返回 True ，那么消息不再传递下去
                break;


        # 后置
        postEvent = notification.post()
        if postEvent:
            for l in self._listeners[postEvent.type()]:
                if l(notification): # 如果返回 True ，那么消息不再传递下去
                    break;


# -*- coding: utf-8 -*-

from enum import Enum
from .notification import Notification
from .notificationcenter import NotificationCenter

class NOTIFICATION(Enum):
    '''
    :brief 预定义事件类型
    '''

    # 吃饭
    EAT = 'eat'
    #睡觉
    SLEEP = 'sleep'

    AFTER_EAT = 'after eat'
    BEFORE_SLEEP = 'before sleep'

# print(type(EVENT.EAT.value))

NotificationEat = Notification(NOTIFICATION.EAT)
NotificationEatPost = Notification(NOTIFICATION.AFTER_EAT)
NotificationEat.before(NotificationEatPost)

NotificationSleep = Notification(NOTIFICATION.SLEEP)
NotificationSleepPre = Notification(NOTIFICATION.BEFORE_SLEEP)
NotificationSleep.after(NotificationSleepPre)

'''
'''

class sleep():
    def register(self):
        eventbus = NotificationCenter()

        eventbus.on(NotificationSleep.type(), self._on_sleep)
        eventbus.on(NotificationSleepPre.type(), self._pre_sleep)

    def _on_sleep(self, event):
        print('开始睡觉了')

    def _pre_sleep(self, event):
        print('睡觉之前')


class eat():
    def register(self):
        eventbus = NotificationCenter()

        eventbus.on(NotificationEat.type(), self._on_eat)
        eventbus.on(NotificationEatPost.type(), self._post_eat)

    def _on_eat(self, event):
        print('开始吃晚饭了')

    def _post_eat(self, event):
        print('饭吃好了，记得多走走')

def test():
    eventbus = NotificationCenter()

    eat().register()
    sleep().register()

    #启动事件驱动
    eventbus.fire(NotificationEat)
    eventbus.fire(NotificationSleep)

if __name__ == '__main__':
    test()


# Insert your code here. 

'''
Usage

@static_vars(counter = 0)
def foo():
    foo.counter += 1
'''
def static_vars(**kwargs):
    def decorate(func):
        for k in kwargs:
            setattr(func, k, kwargs[k])
        return func
    return decorate


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from enum import Enum, unique

@unique
class NOTIFICATION(Enum):
    '''
    :brief 预定义事件类型
    '''
    EXAMPLE = 'example'

# NOTIFICATION.EEEE = 'DDDD';
setattr(NOTIFICATION, 'EEEE', 'DDDD')

class AA_NOTIFICATION(NOTIFICATION):
    XXX = 'ddddddd'

def parse_notification(notification_str):
    return NOTIFICATION.__members__.get(notification_str.upper(), None)

'''
@brief 通知，在当前线程执行
'''
class Notification(object):
    _notification_type = None
    _notification_payload = None

    _before = None
    _after = None

    def __init__(self, notification_type, notification_payload=None):
        self._notification_type = notification_type
        self._notification_payload = notification_payload

    def __repr__(self):
        return ' '.join('{}:{}'.format(k, v) for k, v in self.__dict__.items())

    def before(self, notification):
        '''
        添加后置事件
        :param event:
        :return:
        '''
        self._after = notification

    def after(self, notification):
        '''
        添加前置事件
        :param event:
        :return:
        '''
        self._before = notification

    def type(self):
        '''
        获取事件类型
        :return:
        '''
        return self._notification_type

    def data(self):
        '''
        获取事件负载
        :return:
        '''
        return self._notification_payload

    def pre(self):
        '''
        获取前置事件

        :return:
        '''
        return self._before

    def post(self):
        '''
        获取后置事件
        :return:
        '''
        return self._after


d = parse_notification('ddddddd')
print(str(d))


# -*- coding: utf-8 -*-

###############################################
# - 导入
###############################################


###############################################
# - [关于MVC架构中的Repository模式](http://blog.csdn.net/syaguang2006/article/details/22111211)
# - [项目架构开发：数据访问层之Repository](https://www.cnblogs.com/lanxiaoke/p/6503022.html)
###############################################

class Repository(object):
    ## 创建实例
    # @desc 不做插入动作, db中不应该产生实例
    @staticmethod
    def create():
        pass

    ## 插入实例
    #
    @staticmethod
    def insert(entity):
        pass

    ## 批量插入实例
    #
    @staticmethod
    def insert_batch(entitys):
        pass

    ## 更新实例
    #
    @staticmethod
    def update(entity):
        pass

    ## 删除实例
    #
    @staticmethod
    def delete(entity):
        pass

    ## 删除实例 通过 id
    #
    @staticmethod
    def delete_by_id(id):
        pass

    @staticmethod
    def delete_by_ids(ids):
        pass

    ## 删除实例 通过 uid, user identification 用户识别; user identifier 用户标识符; universal identifier 通用标识符; unique identifier 惟一标识符;
    #
    @staticmethod
    def delete_by_uid(uid):
        pass

    @staticmethod
    def delete_by_uids(uids):
        pass

    ## 查询所有
    #
    @staticmethod
    def query_all():
        pass

    ## 查询 按 id （一般为整型）
    #
    @staticmethod
    def query_by_id(id):
        pass

    @staticmethod
    def query_by_ids(ids):
        pass

    ## 查询 按 uid （一般为字符串标识符）
    #
    @staticmethod
    def query_by_uid(uid):
        pass

    ## 查询 按 匿名函数
    # @param exp: User.id=user_id
    # @return 实体对象数组
    @staticmethod
    def query_by(exp):
        pass

    ## 查询总数目
    #
    @staticmethod
    def count_all():
        pass

    ## 查询 按 匿名函数
    # @return 实体对象数组
    @staticmethod
    def count_by(exp):
        pass

    ## 实例是否存在
    #
    @staticmethod
    def exist_by_id(id):
        pass

    ## 实例是否存在
    #
    @staticmethod
    def exist_by_uid(uid):
        pass

    ## 实例是否存在 按 匿名函数
    # @return BOOL
    @staticmethod
    def exist_by(exp):
        pass



    ####### 基类提供实现的方法
    @staticmethod
    def strip():
        pass
